@@ -51,8 +51,17 @@ module Data.Validity
     , decorateList
     , invalid
     , valid
+    -- ** Helpers for specific types
+    -- *** Char
+    , validateCharNotUtf16SurrogateCodePoint
+    , isUtf16SurrogateCodePoint
+    -- *** RealFloat (Double)
     , validateNotNaN
     , validateNotInfinite
+    -- *** Ratio
+    , validateRatioNotNaN
+    , validateRatioNotInfinite
+    , validateRatioNormalised
     -- * Utilities
     -- ** Utilities for validity checking
     , isValid
@@ -85,6 +94,8 @@ import Data.Maybe (fromMaybe)
 import Data.Monoid
 import Data.Ratio
 #endif
+import Data.Bits ((.&.))
+import Data.Char (ord)
 import Data.Int (Int16, Int32, Int64, Int8)
 #if MIN_VERSION_base(4,8,0)
 import Data.Word (Word16, Word32, Word64, Word8)
@@ -388,6 +399,13 @@ instance Validity Ordering where
 instance Validity Char where
     validate = trivialValidation
 
+validateCharNotUtf16SurrogateCodePoint :: Char -> Validation
+validateCharNotUtf16SurrogateCodePoint c =
+  declare "The character is not a UTF16 surrogate codepoint" $ not $ isUtf16SurrogateCodePoint c
+
+isUtf16SurrogateCodePoint :: Char -> Bool
+isUtf16SurrogateCodePoint c = ord c .&. 0x1ff800 == 0xd800
+
 -- | Trivially valid
 instance Validity Int where
     validate = trivialValidation
@@ -437,10 +455,34 @@ instance Validity Double where
     validate = trivialValidation
 
 validateNotNaN :: RealFloat a => a -> Validation
-validateNotNaN d = declare "The Double is not NaN." $ not (isNaN d)
+validateNotNaN d = declare "The RealFloat is not NaN." $ not (isNaN d)
 
 validateNotInfinite :: RealFloat a => a -> Validation
-validateNotInfinite d = declare "The Double is not infinite." $ not (isInfinite d)
+validateNotInfinite d = declare "The RealFloat is not infinite." $ not (isInfinite d)
+
+validateRatioNotNaN :: Integral a => Ratio a -> Validation
+validateRatioNotNaN r = declare "The Ratio is not NaN." $
+  case r of
+    (0 :% 0) -> False
+    _ -> True
+
+validateRatioNotInfinite :: Integral a => Ratio a -> Validation
+validateRatioNotInfinite r = declare "The Ratio is not infinite." $
+  case r of
+    (1 :% 0) -> False
+    ((-1) :% 0) -> False
+    _ -> True
+
+validateRatioNormalised :: Integral a => Ratio a -> Validation
+validateRatioNormalised (n :% d) = declare "The Ratio is normalised." $
+  case d of
+    0 -> False
+    _ ->
+      let g = gcd n d
+          gcdOverflows = g < 0
+          n' :% d' = (n `quot` g) :% (d `quot` g)
+          valueIsNormalised = n' :% d' == n :% d
+      in not gcdOverflows && valueIsNormalised
 
 -- | Trivially valid
 --
@@ -462,18 +504,12 @@ instance Validity Natural where
 -- | Valid if the contained numbers are valid and the denominator is
 -- strictly positive.
 instance (Validity a, Ord a, Num a, Integral a) => Validity (Ratio a) where
-    validate (n :% d) =
+    validate r@(n :% d) =
         mconcat
             [ annotate n "The numerator"
             , annotate d "The denominator"
             , declare "The denominator is strictly positive." $ d > 0
-            , declare "The ratio is normalised" $
-                case d of
-                  0 -> False
-                  _ ->
-                    let g = gcd n d
-                        n' :% d' = (n `quot` g) :% (d `quot` g)
-                    in n' :% d' == n :% d
+            , validateRatioNormalised r
             ]
 
 -- | Valid according to the contained 'Integer'.
@@ -542,7 +578,7 @@ constructValidUnsafe p =
 -- in the form of a list of 'ValidationChain's, or it returns 'Right' with the
 -- input value, as evidence that it is valid.
 --
--- Note: You map want to use 'prettyValidation' instead, if you want to
+-- Note: You may want to use 'prettyValidation' instead, if you want to
 -- display these 'ValidationChain's to a user.
 checkValidity :: Validity a => a -> Either [ValidationChain] a
 checkValidity a =
